// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       09/01/2012                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com                    ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmels evaluation board				    ****
// *****************************************************************

/*
	FreeRTOS.org V4.7.2 - Copyright (C) 2003-2008 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	+++ http://www.FreeRTOS.org +++
	Documentation, latest information, license and contact details.

	+++ http://www.SafeRTOS.com +++
	A version that is certified for use in safety critical systems.

	+++ http://www.OpenRTOS.com +++
	Commercial support, development, porting, licensing and training services.

	***************************************************************************
*/

// ansi c and atmel drivers includes
#include <stdio.h>
#include <string.h>
#include "board.h"
#include "gpio.h"
#include "spi.h"
#include "navigation.h"
#include "fat.h"
#include "file.h"
#include "ctrl_access.h"
#include "sd_mmc_spi.h"
#include "sdramc.h"
#include "usart.h"
#include "pm.h" // environment header files.
#include "sdramc.h"

// scheduler includes.
#include "FreeRTOS.h"
#include "task.h"
// #include "print_funcs.h" // uncoment to use the debug print functions instead of synth core B printing routines

// synth core B libs
#include "defines.h"
#include "com.h"
#include "midi.h"
#include "timing.h"
#include "adc_converters.h"
#include "console.h"
#include "file_system.h"
#include "rtc_clock.h"
#include "gpio_ports.h"

// TODO
// Disable interupts on the respective module's Close() function


/* uncomment task to test serial port
//
// serial_test_process
//
static void serial_test_process( void *pvParameters ){

	portCHAR uc_aux=45;
	signed portCHAR c_aux=0;


	for( ;; ){

		COM_xSerialPutChar(COM_SERIAL_232,uc_aux, 100 );
		COM_xSerialPutChar(COM_SERIAL_USB,uc_aux,100 );
		COM_xSerialPutChar(COM_SERIAL_MIDI,uc_aux, 200 );

		if (COM_xSerialGetChar( COM_SERIAL_232, &c_aux,2)==pdTRUE){

			// tomamos lo que recibimos por el COM 232 y lo programamos para que se envie por todos los demas COMs
			uc_aux=c_aux;

		}else if (COM_xSerialGetChar( COM_SERIAL_USB, &c_aux,2)==pdTRUE){

			// tomamos lo que recibimos por el COM USB y lo programamos para que se envie por todos los demas COMs
			uc_aux=c_aux;

		}else if (COM_xSerialGetChar( COM_SERIAL_MIDI, &c_aux,2)==pdTRUE){

			// tomamos lo que recibimos por el COM MIDI y lo programamos para que se envie por todos los demas COMs
			// como no tengo consola MIDI para probar el midi lo que hago es puentear MIDI IN y MIDI OUT on el cable de
			// test de MIDI, de manera que cuando hay el puente lo que sale por el MIDI OUT se realimenta en MIDI IN se
			// lee y se reenvia por los demas COMs. Si no hay puente no se hace nada.
			uc_aux=c_aux;

		}else{

			// si no se ha pulsado ninguna tecla incrementamos el caracter
			uc_aux++;

		}

		vTaskDelay(500);

		if (uc_aux>=128){ uc_aux=45; }

	}//for

}//serial_test_process
end uncomment to test serial port */



/* uncomment task to test MIDI IN
//
// midi IN test_process
//
static void midi_in_test_process( void *pvParameters ){
	portCHAR uc_aux;
	portCHAR aux_string[MAX_STRING];
	t_MIDI_event MIDI_event;


	for( ;; ){

		 if (MIDI_dequeue_received_event(&MIDI_event)==1){
			 uc_aux=MIDI_event.code;

			 switch (MIDI_event.code){
				case MIDI_COD_NOTE_OFF:
					sprintf (aux_string,"NOTE OFF - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_NOTE_ON:
					sprintf (aux_string,"NOTE ON - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_AFTER_TOUCH:
					sprintf (aux_string,"AFT.TOUCH - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_CTRL_CHANGE:
					sprintf (aux_string,"CTRL.CHANGE - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_PROGR_CHANGE:
					sprintf (aux_string,"PRG.CHANGE - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_CHAN_PRESURE:
					sprintf (aux_string,"CHN.PRESURE - CHAN %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				case MIDI_COD_PTICH_WHEEL:
					sprintf (aux_string,"PIT.WHEEL: %d - %d %d\r\n",MIDI_event.channel,MIDI_event.value_1 , MIDI_event.value_2);
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

					break;
				default:
					sprintf (aux_string,"UNDEFINED\r\n");
					COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));
					break;
			 }

		 }else{

			 COM_xSerialPutChar(COM_SERIAL_232,'n', 1 );
			 vTaskDelay(300);

		 }//if

	}//for

}//midi_test_process
end uncomment to test MIDI IN*/



/* uncoment task to test MIDI OUT
//
// midi OUT test_process
//
static void midi_out_test_process( void *pvParameters ){
	t_MIDI_event MIDI_event;
	portCHAR c_aux_char=32;


	for( ;; ){

		if (c_aux_char>52){
			c_aux_char=32;
		}else{
			c_aux_char++;
		}

		MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
		MIDI_event.code=MIDI_COD_NOTE_ON;
		MIDI_event.channel=0;
		MIDI_event.value_1=c_aux_char;
		MIDI_event.value_2=64;

		MIDI_enqueue_event_to_send(&MIDI_event);

		vTaskDelay(1000);

		MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
		MIDI_event.code=MIDI_COD_AFTER_TOUCH;
		MIDI_event.channel=0;
		MIDI_event.value_1=c_aux_char;
		MIDI_event.value_2=32;

		MIDI_enqueue_event_to_send(&MIDI_event);

		vTaskDelay(2000);

		MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
		MIDI_event.code=MIDI_COD_NOTE_OFF;
		MIDI_event.channel=0;
		MIDI_event.value_1=c_aux_char;
		MIDI_event.value_2=64;

		MIDI_enqueue_event_to_send(&MIDI_event);

		vTaskDelay(1000);

		COM_xSerialPutChar(COM_SERIAL_232,'s', 1 );

	}//for

}//midi_out_test_process
end uncomment to test MIDI OUT */



/* uncomment task to test ADC converters
//
// ADC CONVERTERS test_process
//
static void adc_converters_test_process( void *pvParameters ){
	portCHAR uc_aux;
	unsigned portSHORT ul_aux;
	portCHAR aux_string[MAX_STRING];

	for( ;; ){

		for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
			ul_aux=ADC_CONVERTERS_read_value(uc_aux);
			// print the read value
			sprintf (aux_string,"CHAN: %d VALUE:%d \r\n",uc_aux,ul_aux);
			COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

		}//for

		vTaskDelay(3000);

	}//for

}//adc_converters_test_process
end uncomment to test ADC converters */



/* uncomment task to test alphanumeric HD4780 LCD
//
// GPIO CONVERTERS test_process
//
static void lcd_test_process( void *pvParameters ){

	LCD_HD44780_init();

	for( ;; ){


		LCD_HD44780_write("DABALE ARROZ A LA ZORRA EL ABAD", 1);
		LCD_HD44780_write("     (c) Tolaemon 2012", 2);

	}//for

}//gpio_test_process
end uncomment to test alphanumeric HD4780 LCD */




/* uncomment task to test MIDI OUT together with ADC
//
// MIDI OUT and ADC CONVERTERS test_process
//
static void midi_out_adc_converters_test_process( void *pvParameters ){
	unsigned  portCHAR uc_aux;
	unsigned portSHORT us_aux1;
	unsigned portSHORT us_aux2;
	portCHAR aux_string[MAX_STRING];
	unsigned portSHORT last_value[ADC_CONVERTERS_NUM_CHANNELS];
	t_MIDI_event MIDI_event;


	// set all "last read valid values" to 0
	for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){
		last_value[uc_aux]=0;
	}//for


	for( ;; ){

		for (uc_aux=0;uc_aux<ADC_CONVERTERS_NUM_CHANNELS;uc_aux++){

			us_aux1=ADC_CONVERTERS_read_value(uc_aux);

			// takes the difference between the last valid read
			// value and the current value of the ADCs
			if (us_aux1>last_value[uc_aux]){
				us_aux2=us_aux1-last_value[uc_aux];
			}else{
				us_aux2=last_value[uc_aux]-us_aux1;
			}//if

			// if the difference is greater than 10 update the value
			// and send a MIDI OUT event with the new value
			if (us_aux2>10){

				last_value[uc_aux]=us_aux1;

				// print the read value
				sprintf (aux_string,"NEW VAL CHAN: %d VALUE:%d \r\n",(int)uc_aux,(int)0x7F&(us_aux1>>3) );
				COM_vSerialPutString(COM_SERIAL_232, (signed portCHAR *) aux_string, (unsigned portSHORT) strlen(aux_string));

				switch (uc_aux){
					case 0:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_NOTE_ON;
						MIDI_event.channel=0;
						MIDI_event.value_1=0x7F&(us_aux1>>3);// divide by 8
						MIDI_event.value_2=100;
						break;

					case 1:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_PTICH_WHEEL;
						MIDI_event.channel=0;
						MIDI_event.value_1=0x3FFF&(us_aux1<<4);// multiply by 4
						MIDI_event.value_2=0;
						break;

					case 2:
						// CHANNEL 2 is not available on the board
						break;

					case 3:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_CTRL_CHANGE;
						MIDI_event.channel=0;
						MIDI_event.value_1=1;
						MIDI_event.value_2=0x7F&(us_aux1>>3);;
						break;

					case 4:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_CTRL_CHANGE;
						MIDI_event.channel=0;
						MIDI_event.value_1=2;
						MIDI_event.value_2=0x7F&(us_aux1>>3);;
						break;
					case 5:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_CTRL_CHANGE;
						MIDI_event.channel=0;
						MIDI_event.value_1=3;
						MIDI_event.value_2=0x7F&(us_aux1>>3);;
						break;
					case 6:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_CTRL_CHANGE;
						MIDI_event.channel=0;
						MIDI_event.value_1=4;
						MIDI_event.value_2=0x7F&(us_aux1>>3);;
						break;

					case 7:
						MIDI_event.type=MIDI_VOICE_MESSAGE_TYPE;
						MIDI_event.code=MIDI_COD_CTRL_CHANGE;
						MIDI_event.channel=0;
						MIDI_event.value_1=5;
						MIDI_event.value_2=0x7F&(us_aux1>>3);;
						break;

					default:
						break;

				}//switch

				MIDI_enqueue_event_to_send(&MIDI_event);

			}//if

		}//for

		// vTaskDelay(200);

	}//for

}
end uncomment task to test MIDI OUT together with ADC  */


void PLL_start(){
	volatile avr32_pm_t *pm = &AVR32_PM;

	// Switch the main clock to OSC0
	pm_switch_to_osc0(pm, FOSC0, OSC0_STARTUP);

	// Setup PLL0 on OSC0, mul=15 ,no divisor, lockcount=16, ie. 12Mhzx16 = 192MHz output
	pm_pll_setup(pm,  // volatile avr32_pm_t *pm
			   0,   // unsigned int pll
			   15,   // unsigned int mul
			   1,   // unsigned int div, Sel Osc0/PLL0 or Osc1/Pll1
			   0,   // unsigned int osc
			   16); // unsigned int lockcount

	// set PLL options to run @ 96 Mhz
	pm_pll_set_option(pm, // volatile avr32_pm_t *pm
					0,  // unsigned int pll
					0,  // unsigned int  pll_freq
					1,  // unsigned int  pll_div2
					0); // unsigned int  pll_wbwdisable
	// Enable PLL0
	pm_pll_enable(pm,0);

	// Wait for PLL0 locked
	pm_wait_for_pll0_locked(pm) ;

	// Setup generic clock number 0 on PLL, with OSC0/PLL0, no divisor
	pm_gc_setup(pm,
			  0,
			  1, // Use Osc (=0) or PLL (=1)
			  0, // Sel Osc0/PLL0 or Osc1/Pll1
			  0,
			  0);

	// Enable Generic clock 0
	pm_gc_enable(pm, 0);

	// set divider to 8 for PBA bus and 2 for PBB
	pm_cksel(pm,1,2,1,0,1,0);

}



int main( void ){

	INTC_init_interrupts();

	// start the crystal oscillator 0 and switch the main clock to it.
	pm_switch_to_osc0(&AVR32_PM, FOSC0, OSC0_STARTUP);

	// initialize the debug USART module.
	// init_dbg_rs232(FOSC0);

	// to initialize the external SDRAM chip.
	sdramc_init(FOSC0);

	// PLL_start();

	// resets vars & resources needed by the different peripherals
	COM_reset();
	MIDI_reset();
	ADC_CONVERTERS_reset();
	CONSOLE_reset();
	RTC_CLOCK_reset();

	// initialize and start following peripherals and resources
	COM_init(COM_SERIAL_232,57600,8,USART_NO_PARITY,USART_1_STOPBIT,USART_NORMAL_CHMODE);
	COM_init(COM_SERIAL_USB,57600,8,USART_NO_PARITY,USART_1_STOPBIT,USART_NORMAL_CHMODE);
	COM_init(COM_SERIAL_MIDI,31250,8,USART_NO_PARITY,USART_1_STOPBIT,USART_NORMAL_CHMODE);

	RTC_CLOCK_init();

	RTC_CLOCK_set_time(22,30,00,4,12,2011);

	CONSOLE_init(DEVICE_TYPE_COM, COM_SERIAL_232);

	MIDI_init(DEVICE_TYPE_COM, COM_SERIAL_MIDI);

	ADC_CONVERTERS_init(200);

	// initialize use GIPIO pins
	// Enable the local bus interface for GPIO.
	gpio_local_init();
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_01);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_02);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_03);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_04);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_05);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_06);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_07);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_08);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_09);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_10);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_11);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_12);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_13);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_14);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_15);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_16);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_17);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_18);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_19);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_20);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_21);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_22);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_23);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_24);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_25);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_26);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_27);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_28);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_29);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_30);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_31);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_32);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_33);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_34);
	gpio_local_enable_pin_output_driver(SCOREB_BUS_IO_35);

	/* uncoment to start COM est task
	xTaskCreate(
			serial_test_process
		,  (const signed portCHAR *)"Serial test process"
		,  configMINIMAL_STACK_SIZE
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	end uncoment to start COM test task */

	/* uncoment to start MIDI IN test task
	xTaskCreate(
			midi_in_test_process
		,  (const signed portCHAR *)"MIDI IN test prc"
		,  1024
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	end uncoment to start COM est task */

	/* uncoment to start MIDI OUT test task
	xTaskCreate(
			midi_out_test_process
		,  (const signed portCHAR *)"MIDI OUT test prc"
		,  1024
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	end uncoment to start MIDI OUT est task */

	/* uncoment to start ADC CONVERTERS test task
	 xTaskCreate(
			adc_converters_test_process
		,  (const signed portCHAR *)"ADCs test prc"
		,  1024
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	 end uncoment to start ADC CONVERTERS test task */

	/* uncoment to start MIDI OUT and ADCs test task
	 xTaskCreate(
			 midi_out_adc_converters_test_process
		,  (const signed portCHAR *)"MIDI ADCs test"
		,  1024
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	 end uncoment to start ADC CONVERTERS test task */

    /* uncomment to start LCD Test task
	 xTaskCreate(
			 lcd_test_process
		,  (const signed portCHAR *)"LCD test"
		,  1024
		,  NULL
		,  TASK_PRIORITY_3
		,  NULL );
	end uncomment to start LCD Test task */

	// Start the scheduler
	vTaskStartScheduler();

	// Will only get here if there was insufficient memory to create the idle task

	return 0;
}





