// *****************************************************************
// ****                      Synth CORE B                       ****
// ****                  www.tolaemon.com/scoreb                ****
// ****                       Source code                       ****
// ****                       13/11/2011                        ****
// ****                     Jordi Bartolome                     ****
// ****                                                         ****
// **** IMPORTANT:                                              ****
// **** Using parts of this code means accepting all conditions ****
// **** exposed on the web: www.tolaemon.com/scoreb             ****
// ****                                                         ****
// **** Parts of this code have been taken from the demo source ****
// **** code of EVK1100 Atmel's evaluation board				****
// *****************************************************************

// ansi c and atmel drivers includes
#include <avr32/io.h>
#include <stdio.h>
#include <string.h>
#include "board.h"
#include "gpio.h"
#include "spi.h"
#include "navigation.h"
#include "fat.h"
#include "file.h"
#include "ctrl_access.h"
#include "sd_mmc_spi.h"
#include "sdramc.h"

// scheduler includes.
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

// synth core B includes
#include "defines.h"
#include "adc_converters.h"
#include "console.h"
#include "com.h"
#include "file_system.h"
#include "rtc_clock.h"

// prviate internal vars

// CONSOLE configuration vars
static struct {
	portCHAR initialized; // true if the console is initialized and running
	portCHAR device_type; // the type of the device where the console is connected
	portCHAR device_number; // with the decive_number where the console is connected
	xTaskHandle console_task_handler; // the handler to the FreeRTOS console task
} console_config;

static Bool cmd;//! flag for a command presence
static U8   cmd_type;//! command number
static Bool first_ls;//! flag for first ls : mount if set
static U8   i_str = 0;//! string length
static U8   cmd_type;//! command number
static Bool first_ls;//! flag for first ls : mount if set

static char cmd_str[10 + 2 * MAX_FILE_PATH_LENGTH];//! string for command
static char par_str1[MAX_FILE_PATH_LENGTH];//! string for first arg
static char par_str2[MAX_FILE_PATH_LENGTH];//! string for second arg
static char str_buff[MAX_FILE_PATH_LENGTH];//! buffer for command line



// Synth Core  21-11-2011  Tolaemon
// Function: resets all the vars used in the console routines
// Receives:
// Returns:
void CONSOLE_reset(){

	console_config.initialized=FALSE;
	console_config.device_type=FALSE;
	console_config.device_number=FALSE;
	console_config.console_task_handler=NULL;

}//CONSOLE_reset



// Synth Core  20-11-2011  Tolaemon
// Function: signed portCHAR CONSOLE_init(portCHAR com_number){
// Function which initializes the CONSOLE communications through the specified device
// Receives:
//    portUCHAR device_type: indicating which kind of device is going to be used to
//	connect the console
//    portUCHAR device_number: with the identifying number of the device used to
//	 connect the console
// Returns:
// 	 1: if the console has been properly initialized
//  -1:if the console could not be initialized
// Note:
//   CONSOLE_reset(...) should have been called at the application start up
void CONSOLE_parse_cmd(void)
{
  U8 i, j;

  // Get command type.
  for (i = 0; cmd_str[i] != ' ' && i < i_str; i++);

  if (i)
  {
    cmd = TRUE;
    // Save last byte
    j = cmd_str[i];
    // Reset vars
    cmd_str[i] = '\0';
    par_str1[0] = '\0';
    par_str2[0] = '\0';

    // Decode command type.
    if      (!strcmp(cmd_str, STR_CD      )) cmd_type = CMD_CD;
    else if (!strcmp(cmd_str, STR_MOUNT   )) cmd_type = CMD_MOUNT;
    else if (!strcmp(cmd_str, STR_FAT     )) cmd_type = CMD_FAT;
    else if (!strcmp(cmd_str, STR_CP      )) cmd_type = CMD_CP;
    else if (!strcmp(cmd_str, STR_LS      )) cmd_type = CMD_LS;
    else if (!strcmp(cmd_str, STR_RM      )) cmd_type = CMD_RM;
    else if (!strcmp(cmd_str, STR_DF      )) cmd_type = CMD_DF;
    else if (!strcmp(cmd_str, STR_MKDIR   )) cmd_type = CMD_MKDIR;
    else if (!strcmp(cmd_str, STR_TOUCH   )) cmd_type = CMD_TOUCH;
    else if (!strcmp(cmd_str, STR_APPEND  )) cmd_type = CMD_APPEND;
    else if (!strcmp(cmd_str, STR_CAT     )) cmd_type = CMD_CAT;
    else if (!strcmp(cmd_str, STR_DISK    )) cmd_type = CMD_NB_DRIVE;
    else if (!strcmp(cmd_str, STR_MARK    )) cmd_type = CMD_SET_ID;
    else if (!strcmp(cmd_str, STR_GOTO    )) cmd_type = CMD_GOTO_ID;
    else if (!strcmp(cmd_str, STR_MV      )) cmd_type = CMD_MV;
    else if (!strcmp(cmd_str, STR_A       )) cmd_type = CMD_MOUNT, par_str1[0] = 'a', par_str1[1] = '\0';
    else if (!strcmp(cmd_str, STR_B       )) cmd_type = CMD_MOUNT, par_str1[0] = 'b', par_str1[1] = '\0';
    else if (!strcmp(cmd_str, STR_C       )) cmd_type = CMD_MOUNT, par_str1[0] = 'c', par_str1[1] = '\0';
    else if (!strcmp(cmd_str, STR_D       )) cmd_type = CMD_MOUNT, par_str1[0] = 'd', par_str1[1] = '\0';
    else if (!strcmp(cmd_str, STR_HELP    )) cmd_type = CMD_HELP;
    else if (!strcmp(cmd_str, STR_FORMAT  )) cmd_type = CMD_FORMAT;
    else if (!strcmp(cmd_str, STR_FORMAT32)) cmd_type = CMD_FORMAT32;
    else if (!strcmp(cmd_str, STR_TEST_SDRAM)) cmd_type = CMD_TEST_SDRAM;
    else if (!strcmp(cmd_str, STR_GET_RTC_SEC)) cmd_type = CMD_GET_RTC_SECS;
    else if (!strcmp(cmd_str, STR_GET_RTC_TIME)) cmd_type = CMD_GET_RTC_TIME;
    else
    {
      // error : command not found
      COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"CMD not found\r\n", 15);
      cmd = FALSE;
    }
    // restore last byte
    cmd_str[i] = j;
  }
  // if command isn't found, display prompt
  if (!cmd)
  {
	COM_vSerialPutString(console_config.device_number,(signed portCHAR *)">", 2);
    return;
  }

  // Get first arg (if any).
  if (++i < i_str)
  {
    j = 0;
    // remove " if used
    if (cmd_str[i] == '"')
    {
      i++;
      for (; cmd_str[i] != '"' && i < i_str; i++, j++)
      {
        par_str1[j] = cmd_str[i];
      }
      i++;
    }
    // get the arg directly
    else
    {
      for(; cmd_str[i] != ' ' && i < i_str; i++, j++)
      {
        par_str1[j] = cmd_str[i];
      }
    }
    // null terminated arg
    par_str1[j] = '\0';
  }

  // Get second arg (if any).
  if (++i < i_str)
  {
    j = 0;
    // remove " if used
    if (cmd_str[i] == '"')
    {
      i++;
      for (; cmd_str[i] != '"' && i < i_str; i++, j++)
      {
        par_str2[j] = cmd_str[i];
      }
      i++;
    }
    // get the arg directly
    else
    {
      for (; cmd_str[i] != ' ' && i < i_str; i++, j++)
      {
        par_str2[j] = cmd_str[i];
      }
    }
    // null terminated arg
    par_str2[j] = '\0';
  }
}



// Synth Core  21-11-2011  Tolaemon
// Function: void CONSOLE_build_cmd(void)
// Receives:
// Returns:
// Note:
void CONSOLE_build_cmd(void){
  portCHAR c_aux;
  portCHAR carray_aux[2];

  // If something new in the USART

  if (COM_xSerialGetChar(console_config.device_number, (signed portCHAR *)&c_aux,0)==pdTRUE){
	carray_aux[0]=c_aux;
	carray_aux[1]='\0';
	COM_vSerialPutString(console_config.device_number,(signed portCHAR *)carray_aux, 2);

    switch (c_aux){

		case CR:
		  // Add LF.
		  COM_vSerialPutString(console_config.device_number,(signed portCHAR *)"\r\n", 2);
		  // Add NUL char.
		  cmd_str[i_str] = '\0';
		  // Decode the command.
		  CONSOLE_parse_cmd();
		  i_str = 0;
		  break;

		// ^c abort cmd.
		case ABORT_CHAR:
		  // Reset command length.
		  i_str = 0;
		  // Display prompt.
		  COM_vSerialPutString(console_config.device_number,(signed portCHAR *)">", 1);
		  break;

		// Backspace.
		case BKSPACE_CHAR:
		  if (i_str > 0)
		  {
			// Replace last char.
			  COM_vSerialPutString(console_config.device_number,(signed portCHAR *)"\b \b", 2);
			// Decraese command length.
			i_str--;
		  }
		  break;

		default:
		  // Append to cmd line.
		  cmd_str[i_str++] = c_aux;
		  break;
    }//switch (c_aux)

  }

}//CONSOLE_build_cmd(void)



// Synth Core  21-11-2011  Tolaemon
// Function: void CONSOLE_append_file(void)
// Receives:
// Returns:
// Note:
void CONSOLE_append_file(void){
  portCHAR c;


  COM_vSerialPutString(console_config.device_number,(signed portCHAR *)"Simple text editor, enter char to append, ^q to exit and save\r\n",60);

  // Wait for ^q to quit.
  while (TRUE)
  {

    // If something new in the USART
    if (COM_xSerialGetChar(console_config.device_number, (signed portCHAR *)&c,0)==pdTRUE){

      // if this is not the quit char
      if (c != QUIT_APPEND)
      {
        // Echo the char.
        // print_char(SHL_USART, c);
        COM_vSerialPutString(console_config.device_number,(signed portCHAR *)"-", 1);
        // Add it to the file.
        file_putc(c);
        // if it is a carriage return.
        if (c == CR)
        {
          // Echo line feed.
          // print_char(SHL_USART, LF);
          COM_vSerialPutString(console_config.device_number,(signed portCHAR *)"\r\n", 2);
          // Add line feed to the file.
          file_putc(LF);
        }
      }
      // Quit char received.
      else
      {
        // Exit the append function.
        break;
      }//if

    }//if

  }//while

}//void CONSOLE_append_file



// Synth Core  26-11-2011  Tolaemon
// Function: void CONSOLE_test_sdram() checks the the external SDRAM
// It writes to each of the RAMs memory positions and then it reads it
// again cecking that read values correspond to the previously written values.
// First of all checks the content of the SDRAM
// Receives:
// Returns:
void CONSOLE_test_sdram(){
	  unsigned long sdram_size,sdram_part, i, noErrors = 0, noOKs = 0;
	  volatile unsigned long *sdram = SDRAM;
	  portCHAR aux_string[MAX_STRING];

	  // Initialize the external SDRAM chip.
	  sdramc_init(FOSC0);

	  // Calculate SDRAM size in words (32 bits).
	  sdram_size = SDRAM_SIZE >> 2; // divide entre 4 puesto que cada posicion tiene 4 bytes realizaremos (SDRAM_SIZE/4) accesos
	  sdram_part = sdram_size;

	  // 1 - prueba del contenido inicial
	  noErrors=0;
	  noOKs=0;

	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"External SDRAM test started:\r\n", (unsigned portSHORT)strlen("External SDRAM test started:\r\n"));

	  sprintf (aux_string,"SDRAM size ( in 32 bits words ) is %ld\r\n",sdram_size);
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

	  // Part 1: checks if the content in the SDRAM is the same that was stored in the previous test
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"Initial check:\r\n", (unsigned portSHORT)strlen("Initial check:\r\n"));
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"Verifying the content before starting test. Please wait ...\r\n", (unsigned portSHORT)strlen("Verifying th content before starting test. Please wait ...\r\n"));

	  for (i=0;i<sdram_part;i++){

		  if (sdram[i]==(i&0xFFFFFFFF)){
			  noOKs++;
		  }else{
			  noErrors++;
		  }//if

	  }//for

	  sprintf (aux_string,"Content before testing is SDRAM ERR: %ld  OK: %ld\r\n",noErrors,noOKs);
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

	  // Part 2: SDRAM writting and reading test
	  noErrors=0;
	  noOKs=0;

	  // llenamos la SDRAM con un valor ( la posicion )
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"Test:\r\n", (unsigned portSHORT)strlen("Test:\r\n"));
	  sprintf (aux_string,"Filling %ld SDRAM's positions of 4 bytes each one. Please wait ...\r\n",sdram_size);
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
	  for (i=0;i<sdram_part;i++){
		  sdram[i]=(i&0xFFFFFFFF);
	  }

	  // verificamos que la posicion de la SDRAM leida conserva el valore previamente escrito
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"Verifying the content of each stored position. Please wait ...\r\n", (unsigned portSHORT)strlen("Verifying the content of each stored position. Please wait ...\r\n"));
	  for (i=0;i<sdram_part;i++){

		  if (sdram[i]==(i&0xFFFFFFFF)){
			  noOKs++;
		  }else{
			  noErrors++;
		  }//if

	  }//for

	  sprintf (aux_string,"Test result is SDRAM ERR: %ld  OK: %ld\r\n",noErrors,noOKs);
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

}//CONSOLE_test_sdram



// Synth Core  21-11-2011  Tolaemon
// Function: void CONSOLE_task( void *pvParameters ){
// Implements the console main task
// Receives:
// Returns:
static void CONSOLE_task( void *pvParameters ){
	  U8 i, j;
	  U16 file_size;
	  Fs_index sav_index;
	  static Fs_index mark_index;
	  const char *part_type;
	  U32 VarTemp;
	  portCHAR aux_string[MAX_STRING];
	  unsigned portLONG ul_aux;
	  unsigned portSHORT ush_aux1;
	  unsigned portSHORT ush_aux2;
	  unsigned portSHORT ush_aux3;
	  unsigned portSHORT ush_aux4;
	  unsigned portSHORT ush_aux5;
	  unsigned portSHORT ush_aux6;

	  // displays welcome message
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_CONSOLE_INFO, (unsigned portSHORT)strlen(MSG_CONSOLE_INFO));

	  sprintf(aux_string,"Compilation %s\r\n",__TIMESTAMP__);
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

	  // Display memory MMC_SD card status
	  // Check SD card unit. Memory devices are identified in ATMEL's file system with their
	  // LUN ( LOGIC UNIT NUMBER ) !!! MMC SD card LUN is defined on LUN_ID_SD_MMC_SPI_MEM
	  if (mem_test_unit_ready(LUN_ID_SD_MMC_SPI_MEM) == CTRL_GOOD){

		  mem_read_capacity(LUN_ID_SD_MMC_SPI_MEM, &VarTemp);
		  sprintf (aux_string,"SD unit detected: %d MBs\r\n",(int)(VarTemp + 1) >> (20 - FS_SHIFT_B_TO_SECTOR) );
		  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

	  }else{

		  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"No SD unit detected\r\n", (unsigned portSHORT)strlen("No unit detected yet\r\n"));

	  }//if

	  // Display the prompt
	  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)">", (unsigned portSHORT)strlen(">"));

	  // reset vars
	  cmd = FALSE;
	  cmd_type = CMD_NONE;
	  first_ls = TRUE;

	  // reset all navigators
	  nav_reset();

	  // always loop
	  while (TRUE){


		// While a usable user command on RS232 isn't received, build it
	    if (!cmd){

	    	  CONSOLE_build_cmd();

	    }else{

	    	// perform the command
			switch (cmd_type){

			  // this is a "mount" command
			  case CMD_MOUNT:

				// Get drive number
				i = par_str1[0] - 'a';
				// If drive doesn't exist
				if (i >= nav_drive_nb()){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number,(signed portCHAR *)MSG_ER_DRIVE, strlen(MSG_ER_DRIVE));

				}else{

				  // Reset all navigators.
				  nav_reset();

				  // Select the desired drive.
				  nav_drive_set(i);

				  // Try to mount it.
				  if (!nav_partition_mount()){

					sprintf (aux_string,"Could not mount\r\n");
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
					// Display error message.
					COM_vSerialPutString(console_config.device_number,(signed portCHAR *)MSG_ER_MOUNT, strlen(MSG_ER_DRIVE));
					// Retry to mount at next "ls".
					first_ls = TRUE;

				  }else{

					sprintf (aux_string,"Drive mounted\r\n");
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
					// Clear flag, no need to remount at next "ls".
					first_ls = FALSE;

				  }//if

				}//if
				break;

			  // this is a "fat" information command
			  case CMD_FAT:

				// Regarding the partition type :
				switch (nav_partition_type()){
					case FS_TYPE_FAT_12:
					  // Display FAT12.
					  part_type = "Drive uses FAT12\r\n";
					  break;
					case FS_TYPE_FAT_16:
					  // Display FAT16.
					  part_type = "Drive uses FAT16\r\n";
					  break;
					case FS_TYPE_FAT_32:
					  // Display FAT32.
					  part_type = "Drive uses FAT32\r\n";
					  break;
					default:
					  // Display error message.
					  part_type = "Drive uses an unknown partition type\r\n";
					  break;
				}//switch
				COM_vSerialPutString(console_config.device_number,(signed portCHAR *)part_type, strlen(part_type));
				break;

			  // this is a "ls" command
			  case CMD_LS:

				// Check if params are correct or mount needed.
				if (nav_drive_get() >= nav_drive_nb() || first_ls){
				  first_ls = FALSE;
				  // Reset navigators .
				  nav_reset();
				  // Use the last drive available as default.
				  nav_drive_set(nav_drive_nb() - 1);
				  // Mount it.
				  nav_partition_mount();
				}//if

				// Get the volume name
				nav_dir_name((FS_STRING)str_buff, MAX_FILE_PATH_LENGTH);
				// Display general informations (drive letter and current path)

				sprintf (aux_string,"Volume is %d\r\n",(char)nav_drive_get() );
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				sprintf (aux_string,"Dir name is %s\r\n",(char*)str_buff);
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				// try to sort items by folders
				if (!nav_filelist_first(FS_DIR)){
				  // Sort items by files
				  nav_filelist_first(FS_FILE);
				}//if

				// display items informations
				sprintf (aux_string,"\tSize (Bytes)\tName\r\n");
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				// reset filelist before to start the listing
				nav_filelist_reset();

				// while an item can be found
				while (nav_filelist_set(0, FS_FIND_NEXT))
				{

				  // Get and display current item informations
				  if (nav_file_isdir()){
					  nav_file_name((FS_STRING)str_buff, MAX_FILE_PATH_LENGTH, FS_NAME_GET, TRUE);
					  sprintf (aux_string,"Dir\t %d \t\t %s \r\n",(int)nav_file_lgt(),str_buff);
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				  }else{
					  nav_file_name((FS_STRING)str_buff, MAX_FILE_PATH_LENGTH, FS_NAME_GET, TRUE);
					  sprintf (aux_string,"\t %d \t\t %s \r\n",(int)nav_file_lgt(),str_buff);
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				  }//if

				}//while

				// display the files number
				sprintf (aux_string," %d files\r\n",nav_filelist_nb(FS_FILE));
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				sprintf (aux_string," %d folders\r\n",nav_filelist_nb(FS_DIR));
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				break;

			  // this is a "cd" command
			  case CMD_CD:

				// get arg1 length
				i = strlen(par_str1);
				// Append the '/' char for the nav_setcwd to enter the chosen directory.
				if (par_str1[i - 1] != '/')
				{
				  par_str1[i] = '/';
				  par_str1[i + 1] = '\0';
				}
				// Try to to set navigator on arg1 folder.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE))
				{
				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_UNKNOWN_FILE));
				}
				break;

			  // this is a "cat" command
			  case CMD_CAT:

				// Try to to set navigator on arg1 file.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE)){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_UNKNOWN_FILE));

				}else{

				  // Open the file.
				  file_open(FOPEN_MODE_R);
				  // While the end isn't reached
				  while (!file_eof()){
					// Display next char from file.
					sprintf (aux_string,"%c",file_getc());
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				  }

				  // Close the file.
				  file_close();
				  sprintf (aux_string,"\r\n");
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				}//if
				break;

			  // this is a "help" command
			  case CMD_HELP:

                // print messge
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_HELP_TITLE, (unsigned portSHORT)strlen(MSG_HELP_TITLE));

				// print compilation
				sprintf(aux_string,"Compilation %s\r\n",__TIMESTAMP__);
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				// Display help on USART
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_HELP, (unsigned portSHORT)strlen(MSG_HELP));

				break;

			  // this is a "mkdir" command
			  case CMD_MKDIR:

				// Create the folder;
				nav_dir_make((FS_STRING)par_str1);
				break;

			  // this is a "touch" command
			  case CMD_TOUCH:

				// Create the file.
				nav_file_create((FS_STRING)par_str1);
				break;

			  // this is a "rm" command
			  case CMD_RM:

				// Save current nav position.
				sav_index = nav_getindex();
				// Try to to set navigator on arg1 folder or file.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE)){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_UNKNOWN_FILE));

				}else if (!nav_file_del(FALSE)){
				  // Try to delete the file
				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_RM, (unsigned portSHORT)strlen(MSG_ER_RM));

				}//if
				// Restore nav position.
				nav_gotoindex(&sav_index);
				break;

			  // this is a "append"  command: Append a char to selected file.
			  case CMD_APPEND:

				// Try to to set navigator on arg1 file.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, TRUE)){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_RM));

				}else{

				  // File exists, open it in append mode
				  file_open(FOPEN_MODE_APPEND);

				  // Append from USART
				  CONSOLE_append_file();

				  // Close the file
				  file_close();

				  // Display a line feed to user
				  sprintf (aux_string,"\r\n");
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));

				}//if
				break;

			  // this is a "disk" command
			  case CMD_NB_DRIVE:

				// Display number of drives.
				sprintf (aux_string,"Nb Drive(s): 0%c\r\n",nav_drive_nb());
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				break;

			  // this is a "mark" command
			  case CMD_SET_ID:

				// get marked index from current navigator location
				mark_index = nav_getindex();
				break;

			  // this is a "goto" command
			  case CMD_GOTO_ID:

				// set navigator to the marked index
				nav_gotoindex(&mark_index);
				break;

			  // this is a "cp" command: Copy file to other location.
			  case CMD_CP:

				// Try to set navigator on arg1 file.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE)){

				  // Display error message.
				 COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_RM));

				}else{

				  // Get name of source to be used as same destination name.
				  nav_file_name((FS_STRING)par_str1, MAX_FILE_PATH_LENGTH, FS_NAME_GET, TRUE);
				  // Get file size.
				  file_size = nav_file_lgtsector();
				  // Mark source.
				  nav_file_copy();
				  // Save current source position.
				  sav_index = nav_getindex();
				  // Goto destination.
				  nav_gotoindex(&mark_index);

				  // Free space check.
				  if (nav_partition_space() > file_size){

					// Paste.
					nav_file_paste_start((FS_STRING)par_str1);
					// Restore previous nav position.
					nav_gotoindex(&sav_index);
					// Performs copy.
					while (nav_file_paste_state(FALSE) == COPY_BUSY);

				  }//if

				  // Restore previous nav position.
				  nav_gotoindex(&sav_index);

				}//if
				break;

			  // this is a "mv" command: Rename file.
			  case CMD_MV:

				// Save current nav position.
				sav_index = nav_getindex();
				// Try to to set navigator on arg1 folder or file.
				if (!nav_setcwd((FS_STRING)par_str1, TRUE, FALSE)){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_UNKNOWN_FILE, (unsigned portSHORT)strlen(MSG_ER_RM));

				}else if (!nav_file_rename((FS_STRING)par_str2)){

				  // Try to rename the file
				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_MV, (unsigned portSHORT)strlen(MSG_ER_MV));

				}//if
				// Restore nav position.
				nav_gotoindex(&sav_index);
				break;

			  // this is a "df" command: Display free space information for all connected drives.
			  case CMD_DF:

				// Save current nav position.
				sav_index = nav_getindex();
				// For all available drives :
				for (i = 0; i < nav_drive_nb(); i++){
				  // Select drive.
				  nav_drive_set(i);

				  // Try to mount.
				  if (nav_partition_mount()){
					sprintf (aux_string,"%s ( %c :) Free Space: %d bytes of %d bytes\r\n",mem_name(i),'A'+i,(int)(nav_partition_freespace() << FS_SHIFT_B_TO_SECTOR),(int)(nav_partition_space() << FS_SHIFT_B_TO_SECTOR));
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
				  }//if

				}//for

				// Restore nav position.
				nav_gotoindex(&sav_index);
				break;

			  // this is a "format" command : Format disk.
			  case CMD_FORMAT:

				// Get disk number.
				i = par_str1[0] - 'a';
				// if drive number isn't valid
				if (i >= nav_drive_nb()){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_DRIVE, (unsigned portSHORT)strlen(MSG_ER_DRIVE));

				}else{

				  // Get the current drive in the navigator.
				  j = nav_drive_get();
				  // Select drive to format.
				  nav_drive_set(i);
				  // If format fails.
				  if (!nav_drive_format(FS_FORMAT_DEFAULT)){

					// Display error message.
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_FORMAT, (unsigned portSHORT)strlen(MSG_ER_FORMAT));
					// Return to the previous.
					nav_drive_set(j);

				  } else if (i == j){

					// Format succeds, if drives is the one we were navigating on
					// Reset the navigators.
					nav_reset();
					// Set current drive.
					nav_drive_set(j);

					// If partition mounting fails.
					if (!nav_partition_mount()){

					  // Display error message.
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_MOUNT, (unsigned portSHORT)strlen(MSG_ER_MOUNT));
					  // this will be the first "ls"
					  first_ls = TRUE;

					}else{

					  // this won't be the first "ls", system is already mounted
					  first_ls = FALSE;

					}//if

				  }//if
				  // Format succeds, restore previous navigator drive.
				  else nav_drive_set(j);
				}
				break;

			  // this is a "format32" command: Format disk as FAT 32 if possible.
			  case CMD_FORMAT32:

				// Get disk number.
				i = par_str1[0] - 'a';
				// if drive number isn't valid
				if (i >= nav_drive_nb()){

				  // Display error message.
				  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_DRIVE, (unsigned portSHORT)strlen(MSG_ER_DRIVE));

				}else{

				  // Get the current drive in the navigator.
				  j = nav_drive_get();
				  // Select drive to format.
				  nav_drive_set(i);

				  // If format fails.
				  if (!nav_drive_format(FS_FORMAT_FAT32)){

					// Display error message.
					COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_FORMAT, (unsigned portSHORT)strlen(MSG_ER_FORMAT));
					// Return to the previous.
					nav_drive_set(j);

				  }else if (i == j){

					// Format succeds, if drives is the one we were navigating on
					// Reset the navigators.
					nav_reset();

					// Set current drive.
					nav_drive_set(j);

					// If partition mounting fails.
					if (!nav_partition_mount()){

					  // Display error message.
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_MOUNT, (unsigned portSHORT)strlen(MSG_ER_MOUNT));

					  // this will be the first "ls"
					  first_ls = TRUE;

					}else{

					  // this won't be the first "ls", system is already mounted
					  first_ls = FALSE;

					}//if

				  }//if
				  // Format succeds, restore previous navigator drive.
				  else {
					  nav_drive_set(j);
				  }//if
				}//if
				break;

			  // this is a SDRAMtest command
			  case CMD_TEST_SDRAM:
					// Create the file.
				    CONSOLE_test_sdram();
					break;

			  // this is a RTC test command
			  case CMD_GET_RTC_SECS:
					// Print seconds
				    ul_aux=RTC_CLOCK_read();
				    // Display a line feed to user
				    sprintf (aux_string,"RTC seconds since 1970 ( Unix time ): %lu\r\n",ul_aux);
				    COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string, (unsigned portSHORT)strlen(aux_string));
					break;

			  case CMD_GET_RTC_TIME:
				  // * year,  * month, * day, * hour, * minute, * second
				  if (RTC_CLOCK_get_time(&ush_aux1,&ush_aux2,&ush_aux3,&ush_aux4,&ush_aux5,&ush_aux6)>0){
					  sprintf (aux_string,"RTC time: %u:%u:%u %u/%u/%u \r\n",ush_aux1,ush_aux2,ush_aux3,ush_aux4,ush_aux5,ush_aux6);
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)aux_string,(unsigned portSHORT)strlen(aux_string));
				  }else{
					  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)"Could not read RTC time\r\n", (unsigned portSHORT)strlen("Could not read RTC time\r\n"));
				  }
				  break;
			  // Unknown command.
			  default:
				// Display error message.
				COM_vSerialPutString(console_config.device_number, (signed portCHAR *)MSG_ER_CMD_NOT_FOUND, (unsigned portSHORT)strlen(MSG_ER_CMD_NOT_FOUND));
				break;
			  }
			  // Reset vars.
			  cmd_type = CMD_NONE;
			  cmd = FALSE;
			  // Display prompt.
			  COM_vSerialPutString(console_config.device_number, (signed portCHAR *)">", (unsigned portSHORT)strlen(">"));

			}//if (!cmd) ...

	  }// while (TRUE)

}//CONSOLE_task



// Synth Core  20-11-2011  Tolaemon
// Function: signed portCHAR CONSOLE_init(portCHAR com_number){
// Function which initializes the CONSOLE communications through the specified device
// Receives:
//    portUCHAR device_type: indicating which kind of device is going to be used to
//	connect the console
//    portUCHAR device_number: with the identifying number of the device used to
//	 connect the console
// Returns:
// 	 1: if the console has been properly initialized
//  -1:if the console could not be initialized
// Note:
//   CONSOLE_reset(...) should have been called at the application start up
signed portCHAR CONSOLE_init(portCHAR device_type, portCHAR device_number){
	signed portCHAR return_value=1;

	FILE_SYS_SD_MMC_resources_init();

	// console only can be connected to COM TYPE devices
	if (device_type==DEVICE_TYPE_COM){

		switch (device_number){

			case COM_SERIAL_232:
			case COM_SERIAL_USB:
				console_config.initialized=TRUE;
				console_config.device_type=device_type;
				console_config.device_number=device_number;
				console_config.console_task_handler=NULL;

				// create the CONSOLE task to start processing the bytes
				xTaskCreate(CONSOLE_task ,  (const signed portCHAR *)"CONSOLE task" , CONSOLE_TASK_STACK_SIZE ,  NULL ,  CONSOLE_TASK_PRIORITY ,&console_config.console_task_handler);

				return_value = 1;
				break;

			case COM_SERIAL_MIDI:
				// console cannot be connected to MIDI port
				return_value = 1;
				break;

		}// switch

	}else{

		return_value = -1;

	}//if

	return return_value;

}//CONSOLE_init



// Synth Core  20-11-2011  Tolaemon
// Function: CONSOLE_close
// Function releases all the CONSOLE resources
// Receives:
// Returns:
// Note:
//   CONSOLE_init(...) should have been called at the application start up
void CONSOLE_close(){

	if (console_config.initialized==TRUE){

		// as we are not going to use CONSOLE, then console task is deleted
		vTaskDelete( console_config.console_task_handler );

		console_config.initialized=FALSE;
		console_config.device_type=FALSE;
		console_config.device_number=FALSE;
		console_config.console_task_handler=NULL;

	}//if

}//CONSOLE_close
